@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: keymap.c,v 1.14 2003/01/17 20:24:57 nalin Exp $"
+#ident "$Id: keymap.c,v 1.21 2003/04/29 16:52:04 nalin Exp $"
 #include "../config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -30,12 +30,22 @@
 #include "termcap.h"
 #ifdef HAVE_NCURSES
 #include <ncurses.h>
+#ifdef HAVE_TERM_H
 #include <term.h>
+#endif
+#define HAVE_CURSES
+#else
+#ifdef HAVE_CURSES
+#include <curses.h>
+#ifdef HAVE_TERM_H
+#include <term.h>
+#endif
 #else
 #ifdef HAVE_TERMCAP
 #include <termcap.h>
 #endif
 #endif
+#endif
 
 #ifdef VTE_DEBUG
 static const char *
@@ -114,6 +124,8 @@ static struct _vte_keymap_entry _vte_keymap_GDK_Tab[] = {
 };
 
 static struct _vte_keymap_entry _vte_keymap_GDK_Return[] = {
+	{cursor_all, keypad_all, fkey_all,
+	 VTE_META_MASK, _VTE_CAP_ESC "\n", 2, NULL},
 	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\n", 1, NULL},
 	{cursor_all, keypad_all, fkey_all, 0, "\r", 1, NULL},
 	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
@@ -140,6 +152,52 @@ static struct _vte_keymap_entry _vte_keymap_GDK_ISO_Left_Tab[] = {
 	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
 };
 
+static struct _vte_keymap_entry _vte_keymap_GDK_slash[] = {
+	{cursor_all, keypad_all, fkey_all,
+	 VTE_META_MASK, _VTE_CAP_ESC "/", 2, NULL},
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\037", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, "/", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+
+static struct _vte_keymap_entry _vte_keymap_GDK_question[] = {
+	{cursor_all, keypad_all, fkey_all,
+	 VTE_META_MASK, _VTE_CAP_ESC "?", 2, NULL},
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\177", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, "?", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+
+/* Various numeric keys enter control characters. */
+static struct _vte_keymap_entry _vte_keymap_GDK_2[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\0", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_3[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\033", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_4[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\034", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_5[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\035", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_6[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\036", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_7[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\037", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+static struct _vte_keymap_entry _vte_keymap_GDK_8[] = {
+	{cursor_all, keypad_all, fkey_all, GDK_CONTROL_MASK, "\177", 1, NULL},
+	{cursor_all, keypad_all, fkey_all, 0, NULL, 0, NULL},
+};
+
 /* Home and End are strange cases because their sequences vary wildly from
  * system to system, or mine's just broken.  But anyway. */
 static struct _vte_keymap_entry _vte_keymap_GDK_Home[] = {
@@ -752,10 +810,20 @@ static struct _vte_keymap_group {
 	{GDK_Home,		_vte_keymap_GDK_Home},
 	{GDK_End,		_vte_keymap_GDK_End},
 	{GDK_Insert,		_vte_keymap_GDK_Insert},
+	{GDK_slash,		_vte_keymap_GDK_slash},
+	{GDK_question,		_vte_keymap_GDK_question},
 	/* GDK_Delete is all handled in code, due to funkiness. */
 	{GDK_Page_Up,		_vte_keymap_GDK_Page_Up},
 	{GDK_Page_Down,		_vte_keymap_GDK_Page_Down},
 
+	{GDK_2,			_vte_keymap_GDK_2},
+	{GDK_3,			_vte_keymap_GDK_3},
+	{GDK_4,			_vte_keymap_GDK_4},
+	{GDK_5,			_vte_keymap_GDK_5},
+	{GDK_6,			_vte_keymap_GDK_6},
+	{GDK_7,			_vte_keymap_GDK_7},
+	{GDK_8,			_vte_keymap_GDK_8},
+
 	{GDK_Up,		_vte_keymap_GDK_Up},
 	{GDK_Down,		_vte_keymap_GDK_Down},
 	{GDK_Right,		_vte_keymap_GDK_Right},
@@ -857,7 +925,7 @@ _vte_keymap_map(guint keyval,
 	enum _vte_fkey_mode fkey_mode;
 	char *cap, *tmp;
 	const char *termcap_special = NULL;
-#if defined(HAVE_NCURSES) || defined(HAVE_TERMCAP)
+#if defined(HAVE_CURSES) || defined(HAVE_TERMCAP)
 	char ncurses_buffer[4096];
 	char ncurses_area[512];
 #endif
@@ -947,8 +1015,22 @@ _vte_keymap_map(guint keyval,
 			}
 			mods &= entries[j].mod_mask;
 		}
-		g_assert((fkey_mode == 0) || (fkey_mode == fkey_all));
-		g_assert(mods == 0);
+		switch (_vte_keymap[i].keyval) {
+		case GDK_2:
+		case GDK_3:
+		case GDK_4:
+		case GDK_5:
+		case GDK_6:
+		case GDK_7:
+		case GDK_8:
+			/* Known non-full-coverage cases. */
+			break;
+		default:
+			/* Everything else we double-check. */
+			g_assert((fkey_mode == 0) || (fkey_mode == fkey_all));
+			g_assert(mods == 0);
+			break;
+		}
 		entries = NULL;
 #endif
 		if (_vte_keymap[i].keyval == keyval) {
@@ -1058,7 +1140,7 @@ _vte_keymap_map(guint keyval,
 			}
 		}
 	}
-#if defined(HAVE_NCURSES) || defined(HAVE_TERMCAP)
+#if defined(HAVE_CURSES) || defined(HAVE_TERMCAP)
 	if (termcap_special != NULL) {
 		tmp = g_strdup(terminal);
 		cap = NULL;
@@ -1082,7 +1164,7 @@ _vte_keymap_map(guint keyval,
 #ifdef VTE_DEBUG
 			if (_vte_debug_on(VTE_DEBUG_KEYBOARD)) {
 				int j;
-#ifdef HAVE_NCURSES
+#ifdef HAVE_CURSES
 				fprintf(stderr, " via ncurses to '");
 #else
 #ifdef HAVE_TERMCAP
@@ -1286,18 +1368,31 @@ _vte_keymap_key_add_key_modifiers(guint keyval,
 	nnormal = g_malloc0(*normal_length + 4);
 	memcpy(nnormal, *normal, *normal_length);
 	if (strlen(nnormal) > 1) {
+		/* Get the offset of the last character. */
 		offset = strlen(nnormal) - 1;
 		if (g_ascii_isdigit(nnormal[offset - 1])) {
+			/* Stuff a semicolon and the modifier in right before
+			 * that last character. */
 			nnormal[offset + 2] = nnormal[offset];
 			nnormal[offset + 1] = modifier + '0';
 			nnormal[offset + 0] = ';';
 			*normal_length += 2;
 		} else {
+#if 0
+			/* Stuff a "1", a semicolon and the modifier in right
+			 * before that last character, matching Xterm. */
 			nnormal[offset + 3] = nnormal[offset];
 			nnormal[offset + 2] = modifier + '0';
 			nnormal[offset + 1] = ';';
 			nnormal[offset + 0] = '1';
 			*normal_length += 3;
+#else
+			/* Stuff the modifier in right before that last
+			 * character, matching what people expect. */
+			nnormal[offset + 1] = nnormal[offset];
+			nnormal[offset + 0] = modifier + '0';
+			*normal_length += 1;
+#endif
 		}
 		g_free(*normal);
 		*normal = nnormal;
